var phidget22 = require('phidget22');

var SERVER_PORT = 5661;

function main() {

	if (process.argv.length != 3) {
		console.log('usage: node Dictionary.js <server address>');
		process.exit(1);
	}
	var hostname = process.argv[2];

	console.log('connecting to:' + hostname);
	var conn = new phidget22.Connection(SERVER_PORT, hostname, { name: 'Server Connection', passwd: '' });
	conn.connect()
		.then(runExample)
		.catch(function (err) {
			console.error('Error running example:', err.message);
			process.exit(1);
		});
}

var allkeys = [];
function getKeys(dict, startKey) {

	return (new Promise(function (resolve, reject) {

		dict.scan(startKey).then(function (keys) {
			resolve(keys);
		}).catch(function (err) {
			reject(err);
		});
	})).then(function (keys) {
		if (keys.length == 0)
			return (allkeys);
		allkeys = allkeys.concat(keys);
		return (getKeys(dict, keys[keys.length - 1]));
	});
}

function scanKeys(dict) {

	getKeys(dict, '').then(function (keys) {
		console.log('----------\nKey-Value Pairs\n----------');
		for (var i in keys) {
			dict.get(keys[i]).then(function (val) {
				console.log(this + ' = ' + val);
				if (i == keys.length - 1)
					console.log('----------\n');
			}.bind(keys[i]));
		}
	})
}

function runExample() {

	var d = new phidget22.Dictionary();

	/*
	 * Dictionaries are virtual phidgets: the Phidget Network Server has a test dictionary create
	 * with label "testdict".
	 */
	d.setDeviceLabel('testdict');

	d.onAttach = function (ch) {
		console.log(ch + ' attached');
	};

	d.onDetach = function (ch) {
		console.log(ch + ' detached');
	};

	d.onAdd = function (key, value) {
		console.log('added:' + key + '=' + value);
	};

	d.onUpdate = function (key, value) {
		console.log('updated:' + key + '=' + value);
	};

	d.onRemove = function (key) {
		console.log('removed:' + key);
	};

	d.open().then(function (ch) {
		console.log('channel open');

		console.log('Setting key1 to: value');
		d.set('key1', 'value')
			.then(function () {
				console.log('Setting key1 to: set2');
				d.set('key1', 'set2')
			}).then(function () {
				console.log('Updating key1 to: updated');
				d.update('key1', 'updated');
			}).then(function () {
				console.log('Removing key1');
				d.remove('key1');
			}).then(function () {
				console.log('Adding key1 with value: added');
				d.add('key1', 'added');
			}).then(function () {
				console.log('Removing key1');
				d.remove('key1');
			}).then(function () {
				scanKeys(d);
			}).then(function () {
				console.log("\nActive demonstration complete.\n");
				console.log("Now would be a good time to try out the events by running a second copy of this");
				console.log("program, or using the control pannel example to edit key-value pairs in the");
				console.log("test dictionary.\n");
			}).catch(function (err) {
				console.error('dictionary error', err);
			});

	}).catch(function (err) {
		console.error('failed to open dictionary', err);
	});
}

if (require.main === module)
	main();
